@@ -105,7 +105,7 @@ enmtools.calibrate <- function(model, recalibrate = FALSE, cuts = 11, env = NA,
   env.test.evaluation <- list()
 
   if(recalibrate == TRUE){
-    recalibrated.model <- CalibratR::calibrate(this.pa, pred.df$prob, evaluate_no_CV_error = FALSE)
+    recalibrated.model <- CalibratR::calibrate(this.pa, pred.df$prob, evaluate_no_CV_error = FALSE, ...)
     preds <- raster::rasterToPoints(model$suitability)
     cal.preds <- CalibratR::predict_calibratR(recalibrated.model$calibration_models, preds[,"layer"])
 
